@@ -3,14 +3,12 @@ from logzero import logger
 from pymitter import EventEmitter
 from autosolveclient.autosolveconstants import AutoSolveConstants
 
+
 class AutoSolve:
 
-    autosolve_constants = AutoSolveConstants()
-
     def __init__(self, params):
+        self.autosolve_constants = AutoSolveConstants()
         self.emitter = EventEmitter()
-        self.routing_key = self.api_key.replace("-", "")
-        self.receiver_queue_name = self.build_prefix_with_credentials(self.autosolve_constants.RECEIVER_QUEUE_NAME)
 
         self.access_token = params["access_token"]
         self.api_key = params["api_key"]
@@ -19,11 +17,22 @@ class AutoSolve:
         self.should_alert_on_cancel = params["should_alert_on_cancel"]
         self.manual_reconnect = params["manual_reconnect"]
 
-        self.account_id = ""
-        self.vhost = ""
+        self.routing_key = self.api_key.replace("-", "")
+        self.send_routing_key = self.access_token.replace("-", "")
+
+        self.account_id = self.access_token.split("-")[0]
+        self.vhost = self.autosolve_constants.VHOST
+
+        self.directExchangeName = self.build_with_account_id(self.autosolve_constants.DIRECT_EXCHANGE)
+        self.fanoutExchangeName = self.build_with_account_id(self.autosolve_constants.FANOUT_EXCHANGE)
+
+        self.receiver_queue_name = self.build_prefix_with_credentials(self.autosolve_constants.RECEIVER_QUEUE_NAME)
+
         self.routing_key_receiver = self.build_prefix_with_credentials(self.autosolve_constants.TOKEN_RECEIVE_ROUTE)
         self.routing_key_cancel = self.build_prefix_with_credentials(self.autosolve_constants.CANCEL_RECEIVE_ROUTE)
 
+        self.token_send_routing_key = self.build_with_access_token(self.autosolve_constants.TOKEN_SEND_ROUTE)
+        self.cancel_send_routing_key = self.build_with_access_token(self.autosolve_constants.CANCEL_SEND_ROUTE)
 
         self.connection = None
         self.channel = None
@@ -58,8 +67,6 @@ class AutoSolve:
             raise AuthException(self.autosolve_constants.INVALID_API_KEY_OR_ACCESS_TOKEN)
         else:
             self.log("Validation complete", self.autosolve_constants.SUCCESS)
-            self.account_id = self.access_token.split("-")[0]
-            self.vhost = self.account_id
             self.consumer_thread.start()
 
     def connect(self):
@@ -75,14 +82,12 @@ class AutoSolve:
         try:
             self.connected = self.establish_connection(parameters) \
                              and self.create_channels() \
-                             and self.declare_exchanges() \
-                             and self.register_all_queues()
+                             and self.register_response_queue()
 
             if self.connected:
                 self.log("Beginning message consumption", self.autosolve_constants.SUCCESS)
                 self.ready = True
                 self.channel.start_consuming()
-                self.cancel_channel.start_consuming()
             else:
                 self.log("Error with RabbitMQ connection, attempting to re-establish", self.autosolve_constants.WARNING)
 
@@ -94,7 +99,13 @@ class AutoSolve:
         self.connection.close()
 
     def establish_connection(self, parameters):
-        self.connection = pika.BlockingConnection(parameters)
+        self.log("Attempting connection establishment", self.autosolve_constants.STATUS)
+
+        try:
+            self.connection = pika.BlockingConnection(parameters)
+        except Exception:
+            self.log("Connection Exception", self.autosolve_constants.ERROR)
+
         self.log("Connection established", self.autosolve_constants.SUCCESS)
         return True
 
@@ -103,43 +114,31 @@ class AutoSolve:
         self.channel = self.connection.channel()
         self.cancel_channel = self.connection.channel()
 
-        self.channel.confirm_delivery()
-        self.cancel_channel.confirm_delivery()
         self.log("Channels created", self.autosolve_constants.SUCCESS)
         return True
 
-    def declare_exchanges(self):
-        self.log("Declaring exchange", self.autosolve_constants.STATUS)
-        self.channel.exchange_declare(exchange=self.autosolve_constants.DIRECT_EXCHANGE, exchange_type='direct',
-                                      durable=True)
-        self.cancel_channel.exchange_declare(exchange=self.autosolve_constants.FANOUT_EXCHANGE, exchange_type='fanout',
-                                             durable=True)
-        return True
-
-    def register_all_queues(self):
-        return self.register_response_queue() and self.register_cancel_queue()
-
     def register_response_queue(self):
-        self.channel.queue_declare(self.receiver_queue_name, durable=self.autosolve_constants.DURABLE,
-                                   auto_delete=self.autosolve_constants.AUTO_DELETE)
-        self.log("Queue " + self.receiver_queue_name + " declared", self.autosolve_constants.SUCCESS)
-        self.log("Binding queue to exchange", self.autosolve_constants.STATUS)
-        self.channel.queue_bind(queue=self.receiver_queue_name,
-                                exchange=self.autosolve_constants.DIRECT_EXCHANGE,
-                                routing_key=self.routing_key_receiver)
-        self.log("Queue binded to exchange " + self.autosolve_constants.DIRECT_EXCHANGE,
-                 self.autosolve_constants.SUCCESS)
-        self.channel.basic_consume(queue=self.receiver_queue_name, auto_ack=self.autosolve_constants.AUTO_ACK,
-                                   on_message_callback=self.on_receiver_message)
-        return True
+        try:
+            self.log("Binding queue to exchange", self.autosolve_constants.STATUS)
+            self.channel.queue_bind(queue=self.receiver_queue_name,
+                                    exchange=self.directExchangeName,
+                                    routing_key=self.routing_key_receiver)
+            self.channel.queue_bind(queue=self.receiver_queue_name,
+                                    exchange=self.directExchangeName,
+                                    routing_key=self.routing_key_cancel)
+            self.log("Queue binded to exchange " + self.directExchangeName,
+                     self.autosolve_constants.SUCCESS)
+            self.channel.basic_consume(queue=self.receiver_queue_name, auto_ack=self.autosolve_constants.AUTO_ACK,
+                                       on_message_callback=self.receive_message)
+            return True
+        except pika.exceptions.ChannelClosedByBroker:
+            self.emitter.emit("AutoSolveError", self.autosolve_constants.CONNECTION_ERROR_INIT)
 
-    def register_cancel_queue(self):
-        self.log("Binding cancel routing to exchange", self.autosolve_constants.STATUS)
-        self.cancel_channel.queue_bind(queue=self.receiver_queue_name,
-                                exchange=self.autosolve_constants.DIRECT_EXCHANGE, routing_key=self.routing_key_cancel)
-        self.cancel_channel.basic_consume(queue=self.receiver_queue_name, auto_ack=self.autosolve_constants.AUTO_ACK,
-                                   on_message_callback=self.on_cancel_message)
-        return True
+    def receive_message(self, channel, method_frame, header_frame, body):
+        if method_frame.routing_key == self.routing_key_receiver:
+            self.on_receiver_message(channel, method_frame, header_frame, body)
+        else:
+            self.on_cancel_message(channel, method_frame, header_frame, body)
 
     def on_receiver_message(self, channel, method_frame, header_frame, body):
         json_string = "".join(chr(x) for x in body)
@@ -153,7 +152,7 @@ class AutoSolve:
 
     def handle_connection_error(self):
         self.connected = False
-        if(self.manual_reconnect):
+        if (self.manual_reconnect):
             self.emitter.emit("AutoSolveError", self.autosolve_constants.CONNECTION_LOST_ERROR)
         else:
             while True:
@@ -176,6 +175,10 @@ class AutoSolve:
         message["apiKey"] = self.api_key
         message_string = json.dumps(message)
         byte_string = message_string.encode()
+        if self.fanoutExchangeName == exchange:
+            return self.cancel_channel.basic_publish(exchange=exchange,
+                                            routing_key=route,
+                                            body=byte_string)
         return self.channel.basic_publish(exchange=exchange,
                                           routing_key=route,
                                           body=byte_string)
@@ -183,37 +186,42 @@ class AutoSolve:
     def send_token_request(self, message):
         self.log("Sending request message for task: " + message['taskId'], self.autosolve_constants.STATUS)
         try:
-            send_result = self.send(message, self.autosolve_constants.TOKEN_SEND_ROUTE, self.autosolve_constants.DIRECT_EXCHANGE)
+            send_result = self.send(message, self.token_send_routing_key,
+                                    self.directExchangeName)
             if send_result is None:
-                self.log("Message with TaskId: " + message['taskId'] + " sent successfully", self.autosolve_constants.SUCCESS)
+                self.log("Message with TaskId: " + message['taskId'] + " sent successfully",
+                         self.autosolve_constants.SUCCESS)
         except Exception as e:
             self.log(e, self.autosolve_constants.WARNING)
-            self.resend(message, self.autosolve_constants.TOKEN_SEND_ROUTE, self.autosolve_constants.DIRECT_EXCHANGE)
+            self.resend(message, self.token_send_routing_key, self.directExchangeName)
 
     def cancel_token_request(self, taskId):
-        message = {"taskId" : taskId, "requireResponse" : False}
-        if self.should_alert_on_cancel:
-            message["requireResponse"] = True
+        message = dict()
+        message["taskId"] = taskId
+        message["responseRequired"] = self.should_alert_on_cancel
 
         self.log("Sending cancel message for task: " + message['taskId'], self.autosolve_constants.STATUS)
         try:
-            send_result = self.send(message, self.autosolve_constants.CANCEL_SEND_ROUTE, self.autosolve_constants.FANOUT_EXCHANGE)
+            send_result = self.send(message, self.cancel_send_routing_key,
+                                    self.fanoutExchangeName)
             if send_result is None:
-                self.log("Message with TaskId: " + message['taskId'] + " sent successfully", self.autosolve_constants.SUCCESS)
+                self.log("Cancel message with TaskId: " + message['taskId'] + " sent successfully",
+                         self.autosolve_constants.SUCCESS)
         except Exception as e:
             self.log(e, self.autosolve_constants.WARNING)
-            self.resend(message, self.autosolve_constants.CANCEL_SEND_ROUTE, self.autosolve_constants.FANOUT_EXCHANGE)
+            self.resend(message, self.cancel_send_routing_key, self.fanoutExchangeName)
 
     def cancel_all_requests(self):
         message = {"apiKey": self.api_key}
         self.log("Sending cancel all request for api key ::  " + message['apiKey'], self.autosolve_constants.STATUS)
         try:
-            send_result = self.send(message, self.autosolve_constants.CANCEL_SEND_ROUTE, self.autosolve_constants.FANOUT_EXCHANGE)
+            send_result = self.send(message, self.cancel_send_routing_key,
+                                    self.fanoutExchangeName)
             if send_result is None:
                 self.log("Cancel all requests sent successfully", self.autosolve_constants.SUCCESS)
         except Exception as e:
             self.log(e, self.autosolve_constants.WARNING)
-            self.resend(message, self.autosolve_constants.CANCEL_SEND_ROUTE, self.autosolve_constants.FANOUT_EXCHANGE)
+            self.resend(message, self.cancel_send_routing_key, self.fanoutExchangeName)
 
     def resend(self, message, route, exchange):
         time.sleep(5)
@@ -221,8 +229,9 @@ class AutoSolve:
         if result is None:
             self.log("Resend attempt successful", self.autosolve_constants.SUCCESS)
         else:
-            self.log("Attempt to resend after wait unsuccessful. Pushing message to backlog", self.autosolve_constants.WARNING)
-            self.add_message_to_backlog({"message" : message, "route" : route, "exchange" : exchange})
+            self.log("Attempt to resend after wait unsuccessful. Pushing message to backlog",
+                     self.autosolve_constants.WARNING)
+            self.add_message_to_backlog({"message": message, "route": route, "exchange": exchange})
 
     def add_message_to_backlog(self, message):
         self.message_backlog.append(message)
@@ -272,6 +281,12 @@ class AutoSolve:
     def build_prefix_with_credentials(self, prefix):
         return prefix + "." + self.account_id + "." + self.routing_key
 
+    def build_with_access_token(self, prefix):
+        return prefix + "." + self.send_routing_key
+
+    def build_with_account_id(self, prefix):
+        return prefix + "." + self.account_id
+
     def log(self, message, type):
         if type == self.autosolve_constants.ERROR:
             logger.error(message)
